'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _utils = require('./utils');

var _rcAlign = require('rc-align');

var _rcAlign2 = _interopRequireDefault(_rcAlign);

var _rcAnimate = require('rc-animate');

var _rcAnimate2 = _interopRequireDefault(_rcAnimate);

var Popup = _react2['default'].createClass({
  displayName: 'Popup',

  propTypes: {
    visible: _react2['default'].PropTypes.bool,
    wrap: _react2['default'].PropTypes.object,
    style: _react2['default'].PropTypes.object,
    onMouseEnter: _react2['default'].PropTypes.func,
    onMouseLeave: _react2['default'].PropTypes.func
  },

  componentDidMount: function componentDidMount() {
    this.rootNode = this.getPopupDomNode();
  },

  // optimize for speed
  shouldComponentUpdate: function shouldComponentUpdate(nextProps) {
    return this.props.visible || nextProps.visible;
  },

  onAlign: function onAlign(popupDomNode, align) {
    var props = this.props;
    var placement = props.placement;
    var prefixCls = props.prefixCls;

    if (placement) {
      var originalClassName = (0, _utils.getToolTipClassByPlacement)(prefixCls, placement);
      var nextClassName = undefined;
      if (placement.points) {
        nextClassName = (0, _utils.getToolTipClassByPlacement)(prefixCls, align);
      } else if (typeof placement === 'string') {
        nextClassName = (0, _utils.getToolTipClassByPlacement)(prefixCls, (0, _utils.fromPointsToPlacement)(align));
      }
      if (nextClassName !== originalClassName) {
        popupDomNode.className = popupDomNode.className.replace(originalClassName, nextClassName);
      }
    }
  },

  getPopupDomNode: function getPopupDomNode() {
    return _react2['default'].findDOMNode(this);
  },

  getTarget: function getTarget() {
    return _react2['default'].findDOMNode(this.props.wrap).firstChild;
  },

  getTransitionName: function getTransitionName() {
    var props = this.props;
    var transitionName = props.transitionName;
    if (!transitionName && props.animation) {
      transitionName = props.prefixCls + '-' + props.animation;
    }
    return transitionName;
  },

  render: function render() {
    var props = this.props;
    var prefixCls = props.prefixCls;
    var placement = props.placement;
    var style = props.style;

    var className = undefined;

    if (props.visible || !this.rootNode) {
      className = (0, _utils.getToolTipClassByPlacement)(prefixCls, props.placement);
      if (props.className) {
        className += ' ' + props.className;
      }
    } else {
      // fix auto adjust
      className = this.rootNode.className;
      var hiddenClass = prefixCls + '-hidden';
      if (className.indexOf(hiddenClass) === -1) {
        className += ' ' + hiddenClass;
      }
    }
    var arrowClassName = prefixCls + '-arrow';
    var innerClassname = prefixCls + '-inner';
    var align = undefined;
    if (placement && placement.points) {
      align = placement;
    } else {
      align = _utils.placementAlignMap[placement];
    }

    return _react2['default'].createElement(
      _rcAnimate2['default'],
      { component: '',
        exclusive: true,
        transitionAppear: true,
        transitionName: this.getTransitionName(),
        showProp: 'data-visible' },
      _react2['default'].createElement(
        _rcAlign2['default'],
        { target: this.getTarget,
          key: 'popup',
          monitorWindowResize: true,
          'data-visible': props.visible,
          disabled: !props.visible,
          align: align,
          onAlign: this.onAlign },
        _react2['default'].createElement(
          'div',
          { className: className,
            onMouseEnter: props.onMouseEnter,
            onMouseLeave: props.onMouseLeave,
            style: style },
          _react2['default'].createElement('div', { className: arrowClassName }),
          _react2['default'].createElement(
            'div',
            { className: innerClassname },
            props.children
          )
        )
      )
    );
  }
});

exports['default'] = Popup;
module.exports = exports['default'];